/**
 * @author Kuitos
 * @homepage https://github.com/kuitos/
 * @since 2018-08-15 11:37
 */
import processTpl, { genLinkReplaceSymbol } from './process-tpl';
import { getGlobalProp, noteGlobalProps } from './utils';
export function getDomain(url) {
  try {
    // URL 构造函数不支持使用 // 前缀的 url
    var href = new URL(url.startsWith('//') ? "".concat(location.protocol).concat(url) : url);
    return href.origin;
  } catch (e) {
    return '';
  }
}
var styleCache = {};
var scriptCache = {};
var embedHTMLCache = {};
export default function importHTML(url) {
  var domain = getDomain(url);
  return embedHTMLCache[url] || (embedHTMLCache[url] = fetch(url).then(function (response) {
    return response.text();
  }).then(function (html) {
    var _processTpl = processTpl(html, domain),
        template = _processTpl.template,
        scripts = _processTpl.scripts,
        entry = _processTpl.entry,
        styles = _processTpl.styles;

    function getEmbedHTML() {
      var emberHTML = template;
      return getExternalStyleSheets().then(function (styleSheets) {
        emberHTML = styles.reduce(function (html, styleSrc, i) {
          html = html.replace(genLinkReplaceSymbol(styleSrc), "<style>/* ".concat(styleSrc, " */").concat(styleSheets[i], "</style>"));
          return html;
        }, emberHTML);
        return emberHTML;
      });
    }

    function getExternalStyleSheets() {
      return styleCache[url] || (styleCache[url] = Promise.all(styles.map(function (styleLink) {
        return fetch(styleLink).then(function (response) {
          return response.text();
        });
      })));
    }

    function getExternalScripts() {
      return scriptCache[url] || (scriptCache[url] = Promise.all(scripts.map(function (script) {
        return fetch(script).then(function (response) {
          return response.text();
        });
      })));
    }

    function execScripts() {
      var proxy = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : window;
      return getExternalScripts().then(function (scriptsText) {
        window.proxy = proxy;
        var geval = eval;

        function exec(scriptSrc, inlineScript, nextTick, resolve) {
          var markName = "Evaluating script ".concat(scriptSrc);
          var measureName = "Evaluating Time Consuming: ".concat(scriptSrc);

          if (process.env.NODE_ENV === 'development') {
            performance.mark(markName);
          }

          if (scriptSrc === entry) {
            noteGlobalProps();
            geval(";(function(window){;".concat(inlineScript, "})(window.proxy);"));
            var exports = proxy[getGlobalProp()] || {};
            resolve(exports);
          } else {
            geval(";(function(window){;".concat(inlineScript, "})(window.proxy);"));
          }

          if (process.env.NODE_ENV === 'development') {
            performance.measure(measureName, markName);
            performance.clearMarks(markName);
            performance.clearMeasures(measureName);
          }

          nextTick();
        }

        function schedule(i, resolve) {
          if (i < scripts.length) {
            var scriptSrc = scripts[i];
            var inlineScript = scriptsText[i];
            var nextTick = schedule.bind(null, ++i, resolve);
            var execScriptBinding = exec.bind(null, scriptSrc, inlineScript, nextTick, resolve); // 约 2M 以上的代码延迟执行

            if (inlineScript.length / 1000 / 1000 > 2) {
              // 把时间放出去，确保大文件的执行是完全处于浏览器闲置状态
              setTimeout(execScriptBinding, 200);
            } else {
              window.requestIdleCallback(execScriptBinding);
            }
          }
        }

        return new Promise(function (resolve) {
          return schedule(0, resolve);
        });
      });
    }

    return getEmbedHTML().then(function (embedHTML) {
      return {
        template: embedHTML,
        getExternalScripts: getExternalScripts,
        getExternalStyleSheets: getExternalStyleSheets,
        execScripts: execScripts
      };
    });
  }));
}
;